@@ -543,7 +543,7 @@ void galera::ist::Receiver::run()
             }
         }
 
-        progress->finish();
+        if (progress /* IST actually started */) progress->finish();
     }
     catch (asio::system_error& e)
     {
@@ -561,7 +561,6 @@ void galera::ist::Receiver::run()
     }
 
 err:
-    gcache_.seqno_unlock();
     delete progress;
     gu::Lock lock(mutex_);
     if (use_ssl_ == true)
