@@ -406,73 +406,92 @@ void ReplicatorSMM::process_state_req(void*       recv_ctx,
     wsrep_seqno_t rcode (0);
     bool join_now = true;
 
-    if (streq->ist_len())
+    if (not skip_sst)
     {
-        IST_request istr;
-        get_ist_request(streq, &istr);
-
-        if (istr.uuid() == state_uuid_ && istr.last_applied() >= 0)
+        struct sgl
         {
-            log_info << "IST request: " << istr;
+            gcache::GCache& gcache_;
+            bool            unlock_;
 
-            try
+            sgl(gcache::GCache& cache) : gcache_(cache), unlock_(false){}
+            ~sgl() { if (unlock_) gcache_.seqno_unlock(); }
+        }
+        seqno_lock_guard(gcache_);
+
+        if (streq->ist_len())
+        {
+            IST_request istr;
+            get_ist_request(streq, &istr);
+
+            if (istr.uuid() == state_uuid_ && istr.last_applied() >= 0)
             {
-                gcache_.seqno_lock(istr.last_applied() + 1);
-            }
-            catch(gu::NotFound& nf)
-            {
-                log_info << "IST first seqno " << istr.last_applied() + 1
-                         << " not found from cache, falling back to SST";
-                // @todo: close IST channel explicitly
-                goto full_sst;
-            }
+                log_info << "IST request: " << istr;
 
-            if (streq->sst_len()) // if joiner is waiting for SST, notify it
-            {
-                wsrep_gtid_t const state_id =
-                    { istr.uuid(), istr.last_applied() };
-
-                rcode = donate_sst(recv_ctx, *streq, state_id, true);
-
-                // we will join in sst_sent.
-                join_now = false;
-            }
-
-            if (rcode >= 0)
-            {
                 wsrep_seqno_t const first
                     ((str_proto_ver < 3 || cc_lowest_trx_seqno_ == 0) ?
-                     istr.last_applied() + 1 :
-                     std::min(cc_lowest_trx_seqno_, istr.last_applied()+1));
+                    istr.last_applied() + 1 :
+                    std::min(cc_lowest_trx_seqno_, istr.last_applied()+1));
+
                 try
                 {
-                    ist_senders_.run(config_,
-                                     istr.peer(),
-                                     first,
-                                     cc_seqno_,
-                                     cc_lowest_trx_seqno_,
-                                     /* Historically IST messages versioned
-                                      * with the global replicator protocol.
-                                      * Need to keep it that way for backward
-                                      * compatibility */
-                                     protocol_version_);
+                    gcache_.seqno_lock(first);
+                    seqno_lock_guard.unlock_ = true;
                 }
-                catch (gu::Exception& e)
+                catch(gu::NotFound& nf)
                 {
-                    log_error << "IST failed: " << e.what();
-                    rcode = -e.get_errno();
+                    log_info << "IST first seqno " << istr.last_applied() + 1
+                             << " not found from cache, falling back to SST";
+                    // @todo: close IST channel explicitly
+                    goto full_sst;
                 }
-            }
-            else
-            {
-                log_error << "Failed to bypass SST";
-            }
 
-            goto out;
+                if (streq->sst_len()) // if joiner is waiting for SST, notify it
+                {
+                    wsrep_gtid_t const state_id =
+                        { istr.uuid(), istr.last_applied() };
+
+                    rcode = donate_sst(recv_ctx, *streq, state_id, true);
+
+                    // we will join in sst_sent.
+                    join_now = false;
+                }
+
+                if (rcode >= 0)
+                {
+                    try
+                    {
+                        ist_senders_.run(config_,
+                                         istr.peer(),
+                                         first,
+                                         cc_seqno_,
+                                         cc_lowest_trx_seqno_,
+                                         /* Historically IST messages versioned
+                                          * with the global replicator protocol.
+                                          * Need to keep it that way for backward
+                                          * compatibility */
+                                         protocol_version_);
+                        // seqno will be unlocked when sender exists
+                        seqno_lock_guard.unlock_ = false;
+                    }
+                    catch (gu::Exception& e)
+                    {
+                        log_error << "IST failed: " << e.what();
+                        rcode = -e.get_errno();
+                    }
+                }
+                else
+                {
+                    log_error << "Failed to bypass SST";
+                }
+
+                goto out;
+            }
         }
 
     full_sst:
 
+        assert(!seqno_lock_guard.unlock_);
+
         if (cert_.nbo_size() > 0)
         {
             log_warn << "Non-blocking operation in progress, cannot donate SST";
@@ -516,6 +535,7 @@ void ReplicatorSMM::process_state_req(void*       recv_ctx,
                         }
 
                         gcache_.seqno_lock(preload_start);
+                        seqno_lock_guard.unlock_ = true;
                     }
                     catch (gu::NotFound& nf)
                     {
@@ -543,6 +563,8 @@ void ReplicatorSMM::process_state_req(void*       recv_ctx,
                                       * Need to keep it that way for backward
                                       * compatibility */
                                      protocol_version_);
+                    // seqno will be unlocked when sender exists
+                    seqno_lock_guard.unlock_ = false;
                 }
                 else /* streq->version() == 0 */
                 {
@@ -551,12 +573,9 @@ void ReplicatorSMM::process_state_req(void*       recv_ctx,
                 }
             }
 
-            if (not skip_sst)
-            {
-                rcode = donate_sst(recv_ctx, *streq, state_id, false);
-                // we will join in sst_sent.
-                join_now = false;
-            }
+            rcode = donate_sst(recv_ctx, *streq, state_id, false);
+            // we will join in sst_sent.
+            join_now = false;
         }
         else
         {
